@@ -20,6 +20,7 @@
 #'   removed. See [collinear()].
 #' @param type Type of vector returned. Default "index" returns indices, "names"
 #'   returns predictor names, "full" returns a matrix of p values.
+#' @param ... optional arguments, e.g. `rsq_method`: see [collinear()].
 #'
 #' @return Integer vector of indices of filtered parameters (type = "index") or
 #'   character vector of names (type = "names") of filtered parameters in order
@@ -46,12 +47,13 @@
 #' @export
 #'
 ttest_filter <- function(y,
-                       x,
-                       force_vars = NULL,
-                       nfilter = NULL,
-                       p_cutoff = 0.05,
-                       rsq_cutoff = NULL,
-                       type = c("index", "names", "full")) {
+                         x,
+                         force_vars = NULL,
+                         nfilter = NULL,
+                         p_cutoff = 0.05,
+                         rsq_cutoff = NULL,
+                         type = c("index", "names", "full"),
+                         ...) {
   type <- match.arg(type)
   y <- factor(y)
   indx1 <- as.numeric(y) == 1
@@ -62,12 +64,12 @@ ttest_filter <- function(y,
   rownames(res) <- colnames(x)
   if (type == "full") return(res)
   filter_end(res[, "pvalue"],
-                x, force_vars, nfilter, p_cutoff, rsq_cutoff, type)
+             x, force_vars, nfilter, p_cutoff, rsq_cutoff, type, ...)
 }
 
 
 filter_end <- function(pval, x, force_vars, nfilter, p_cutoff, rsq_cutoff,
-                          type) {
+                          type, ...) {
   check_vars <- which(!colnames(x) %in% force_vars)
   outp <- pval[check_vars]
   outorder <- order(outp)
@@ -75,12 +77,13 @@ filter_end <- function(pval, x, force_vars, nfilter, p_cutoff, rsq_cutoff,
   outp <- outp[outorder]
   if (!is.null(p_cutoff)) out <- out[outp < p_cutoff]
   if (!is.null(rsq_cutoff)) {
-    co <- collinear(x[, out], rsq_cutoff = rsq_cutoff)
+    co <- collinear(x[, out], rsq_cutoff = rsq_cutoff, ...)
     if (length(co) > 0) out <- out[-co]
   }
   if (!is.null(nfilter)) out <- out[1:min(nfilter, length(out))]
   if (length(out) == 0) stop("No predictors left after filtering")
   out <- c(which(colnames(x) %in% force_vars), out)
+  out <- out[!is.na(out)]
   switch(type,
          index = out, names = colnames(x)[out])
 }
@@ -106,6 +109,7 @@ filter_end <- function(pval, x, force_vars, nfilter, p_cutoff, rsq_cutoff,
 #'   removed. See [collinear()].
 #' @param type Type of vector returned. Default "index" returns indices,
 #' "names" returns predictor names, "full" returns a matrix of p values.
+#' @param ... optional arguments, e.g. `rsq_method`: see [collinear()].
 #' @return Integer vector of indices of filtered parameters (type = "index") or 
 #' character vector of names (type = "names") of filtered parameters. If 
 #' `type` is `"full"` full output from [Rfast::ftests] is returned.
@@ -127,7 +131,8 @@ anova_filter <- function(y,
                          nfilter = NULL,
                          p_cutoff = 0.05,
                          rsq_cutoff = NULL,
-                         type = c("index", "names", "full")) {
+                         type = c("index", "names", "full"),
+                         ...) {
   type <- match.arg(type)
   y <- factor(y)
   x <- as.matrix(x)
@@ -135,7 +140,7 @@ anova_filter <- function(y,
   rownames(res) <- colnames(x)
   if (type == "full") return(res)
   filter_end(res[, "pval"],
-                x, force_vars, nfilter, p_cutoff, rsq_cutoff, type)
+             x, force_vars, nfilter, p_cutoff, rsq_cutoff, type, ...)
 }
 
 
@@ -157,6 +162,9 @@ anova_filter <- function(y,
 #'   on Wilcoxon test. If 2 or more predictors are collinear, the first ranked
 #'   predictor by Wilcoxon test is retained, while the other collinear predictors are
 #'   removed. See [collinear()].
+#' @param rsq_method character string indicating which correlation coefficient
+#'   is to be computed. One of "pearson" (default), "kendall", or "spearman".
+#'   See [collinear()].
 #' @param type Type of vector returned. Default "index" returns indices,
 #' "names" returns predictor names, "full" returns a matrix of p-values.
 #' @param exact Logical whether exact or approximate p-value is calculated. 
@@ -175,6 +183,7 @@ wilcoxon_filter <- function(y,
                             nfilter = NULL,
                             p_cutoff = 0.05,
                             rsq_cutoff = NULL,
+                            rsq_method = "pearson",
                             type = c("index", "names", "full"),
                             exact = FALSE,
                             ...) {
@@ -188,7 +197,8 @@ wilcoxon_filter <- function(y,
   )
   if (type == "full") return(res)
   filter_end(res[, "pvalue"],
-                x, force_vars, nfilter, p_cutoff, rsq_cutoff, type)
+             x, force_vars, nfilter, p_cutoff, rsq_cutoff, type,
+             rsq_method = rsq_method)
 }
 
 
@@ -212,8 +222,8 @@ wilcoxon_filter <- function(y,
 #' @export
 #' 
 correls2 <- function(y, x,
-                    method = "pearson",
-                    use = "complete.obs") {
+                     method = "pearson",
+                     use = "complete.obs") {
   ok <- complete.cases(x, y)
   x <- x[ok, ]
   y <- y[ok]
@@ -287,18 +297,18 @@ correl_filter <- function(y,
 #' @param type Type of vector returned. Default "index" returns indices,
 #' "names" returns predictor names, "full" returns a named vector of variable 
 #' importance.
-#' @param ntree Number of trees to grow. See [randomForest].
+#' @param ntree Number of trees to grow. See [randomForest::randomForest].
 #' @param mtry Number of predictors randomly sampled as candidates at each 
-#' split. See [randomForest].
-#' @param ... Optional arguments passed to [randomForest].
+#' split. See [randomForest::randomForest].
+#' @param ... Optional arguments passed to [randomForest::randomForest].
 #' @return Integer vector of indices of filtered parameters (type = "index") or 
 #' character vector of names (type = "names") of filtered parameters. If 
 #' `type` is `"full"` a named vector of variable importance is returned.
 #' @details
-#' This filter uses the [randomForest] function from the randomForest package.
-#' Variable importance is calculated using the [importance] function, specifying
-#' type 1 = mean decrease in accuracy. See [importance].
-#' @importFrom randomForest randomForest importance
+#' This filter uses the `randomForest()` function from the `randomForest`
+#' package. Variable importance is calculated using the
+#' [randomForest::importance] function, specifying type 1 = mean decrease in
+#' accuracy. See [randomForest::importance].
 #' @export
 #' 
 rf_filter <- function(y, x, nfilter = NULL,
@@ -306,6 +316,10 @@ rf_filter <- function(y, x, nfilter = NULL,
                       ntree = 1000,
                       mtry = ncol(x) * 0.2,
                       ...) {
+  if (!requireNamespace("randomForest", quietly = TRUE)) {
+    stop("Package 'randomForest' must be installed to use this filter",
+         call. = FALSE)
+  }
   type <- match.arg(type)
   fit <- randomForest::randomForest(x, y, importance = TRUE,
                                     ntree = ntree, mtry = mtry, ...)
@@ -320,6 +334,58 @@ rf_filter <- function(y, x, nfilter = NULL,
   out
 }
 
+
+#' Random forest ranger filter
+#' 
+#' Fits a random forest model via the `ranger` package and ranks variables by
+#' variable importance.
+#' 
+#' @param y Response vector
+#' @param x Matrix of predictors
+#' @param nfilter Number of predictors to return. If `NULL` all predictors are 
+#' returned.
+#' @param type Type of vector returned. Default "index" returns indices,
+#' "names" returns predictor names, "full" returns a named vector of variable 
+#' importance.
+#' @param num.trees Number of trees to grow. See [ranger::ranger].
+#' @param mtry Number of predictors randomly sampled as candidates at each 
+#' split. See [ranger::ranger].
+#' @param ... Optional arguments passed to [ranger::ranger].
+#' @return Integer vector of indices of filtered parameters (type = "index") or 
+#' character vector of names (type = "names") of filtered parameters. If 
+#' `type` is `"full"` a named vector of variable importance is returned.
+#' @details
+#' This filter uses the `ranger()` function from the `ranger` package. Variable
+#' importance is calculated using mean decrease in gini impurity.
+#' @export
+#' 
+ranger_filter <- function(y, x, nfilter = NULL,
+                          type = c("index", "names", "full"),
+                          num.trees = 1000,
+                          mtry = ncol(x) * 0.2,
+                          ...) {
+  if (!requireNamespace("ranger", quietly = TRUE)) {
+    stop("Package 'ranger' must be installed to use this filter",
+         call. = FALSE)
+  }
+  type <- match.arg(type)
+  fit <- ranger::ranger(x = x, y = y, importance = "impurity",
+                        num.trees = num.trees, mtry = mtry,
+                        write.forest = FALSE,
+                        verbose = FALSE,
+                        num.threads = 1, ...)
+  vi <- fit$variable.importance
+  names(vi) <- if (type == "index") 1:ncol(x) else colnames(x)
+  if (type == "full") return(vi)
+  vi <- sort(vi, decreasing = TRUE)
+  vi <- vi[vi != 0]
+  if (!is.null(nfilter)) vi <- vi[1:min(nfilter, length(vi))]
+  out <- names(vi)
+  if (type == "index") out <- as.integer(out)
+  out
+}
+
+
 #' ReliefF filter
 #' 
 #' Uses ReliefF algorithm from the CORElearn package to rank predictors in order 
@@ -337,13 +403,16 @@ rf_filter <- function(y, x, nfilter = NULL,
 #' @return Integer vector of indices of filtered parameters (type = "index") or 
 #' character vector of names (type = "names") of filtered parameters. If 
 #' `type` is `"full"` a named vector of variable importance is returned.
-#' @seealso [CORElearn::attrEval]
-#' @importFrom CORElearn attrEval
+#' @seealso [CORElearn::attrEval()]
 #' @export
 #'
 relieff_filter <- function(y, x, nfilter = NULL, 
                            estimator = "ReliefFequalK",
                            type = c("index", "names", "full"), ...) {
+  if (!requireNamespace("CORElearn", quietly = TRUE)) {
+    stop("Package 'CORElearn' must be installed to use this filter",
+         call. = FALSE)
+  }
   type <- match.arg(type)
   df <- as.data.frame(x)
   df$y <- y
@@ -395,11 +464,17 @@ combo_filter <- function(y, x,
 
 #' glmnet filter
 #'
-#' Filter using properties of elastic net regression using glmnet to calculate
+#' Filter using sparsity of elastic net regression using glmnet to calculate
 #' variable importance.
 #'
 #' @param y Response vector
 #' @param x Matrix of predictors
+#' @param family Either a character string representing one of the built-in
+#'   families, or else a `glm()` family object. See [glmnet()]. If not
+#'   specified, the function tries to set this automatically to one of either
+#'   "gaussian", "binomial" or "multinomial".
+#' @param force_vars Vector of column names `x` which have no shrinkage and are
+#'   always included in the model.
 #' @param nfilter Number of predictors to return
 #' @param method String indicating method of determining variable importance.
 #'   "mean" (the default) uses the mean absolute coefficients across the range
@@ -422,13 +497,25 @@ combo_filter <- function(y, x,
 #' 
 glmnet_filter <- function(y,
                           x,
+                          family = NULL,
+                          force_vars = NULL,
                           nfilter = NULL,
                           method = c("mean", "nonzero"),
                           type = c("index", "names", "full"),
                           ...) {
   type <- match.arg(type)
   method <- match.arg(method)
-  fit <- glmnet(x, y, ...)
+  if (is.null(family)) {
+    family <- if (is.factor(y) | is.character(y)) {
+      if (nlevels(factor(y)) == 2) "binomial" else "multinomial"
+    } else "gaussian"
+  }
+  penalty.factor <- rep(1, ncol(x))
+  if (!is.null(force_vars)) {
+    keep <- which(colnames(x) %in% force_vars)
+    penalty.factor[keep] <- 0
+  } else keep <- NULL
+  fit <- glmnet(x, y, family = family, penalty.factor = penalty.factor, ...)
   cf <- as.matrix(coef(fit))
   if (method == "mean") {
     cf <- abs(cf)
@@ -440,8 +527,10 @@ glmnet_filter <- function(y,
   out <- out[-1]  # remove intercept
   if (type == "full") return(out)
   if (type == "index") names(out) <- 1:ncol(x)
+  if (!is.null(force_vars)) {
+    out <- out[c(keep, order(out[-keep], decreasing = TRUE))]
+  } else out <- sort(out, decreasing = TRUE)
   out <- out[out != 0]
-  out <- sort(out, decreasing = TRUE)
   if (!is.null(nfilter)) out <- out[1:min(nfilter, length(out))]
   out <- names(out)
   if (length(out) == 0) stop("No predictors selected")
@@ -449,6 +538,7 @@ glmnet_filter <- function(y,
   out
 }
 
+
 # Code modified from caret::findCorrelation
 # https://github.com/topepo/caret/blob/master/pkg/caret/R/findCorrelation.R
 
@@ -464,13 +554,17 @@ glmnet_filter <- function(y,
 #'   determine which columns to retain, so the columns in `x` should be sorted
 #'   with the most important columns first.
 #' @param rsq_cutoff Value of cut-off for r-squared
+#' @param rsq_method character string indicating which correlation coefficient
+#'   is to be computed. One of "pearson" (default), "kendall", or "spearman".
+#'   See [cor()].
 #' @param verbose Boolean whether to print details
 #' @return Integer vector of the indices of columns in `x` to remove due to
 #'   collinearity
 #' @export
 #' 
-collinear <- function(x, rsq_cutoff = 0.9, verbose = FALSE) {
-  rsq <- cor(x)^2
+collinear <- function(x, rsq_cutoff = 0.9, rsq_method = "pearson",
+                      verbose = FALSE) {
+  rsq <- cor(x, method = rsq_method)^2
   rsq[lower.tri(rsq, diag = TRUE)] <- NA
   combsAboveCutoff <- which(rsq > rsq_cutoff)
   colsToCheck <- ceiling(combsAboveCutoff / nrow(rsq))
@@ -522,6 +616,9 @@ collinear <- function(x, rsq_cutoff = 0.9, verbose = FALSE) {
 #'   on AIC from a linear model. If 2 or more predictors are collinear, the
 #'   first ranked predictor by AIC is retained, while the other collinear
 #'   predictors are removed. See [collinear()].
+#' @param rsq_method character string indicating which correlation coefficient
+#'   is to be computed. One of "pearson" (default), "kendall", or "spearman".
+#'   See [collinear()].
 #' @param type Type of vector returned. Default "index" returns indices, "names"
 #'   returns predictor names, "full" returns a matrix of p values.
 #' @return Integer vector of indices of filtered parameters (`type = "index"`)
@@ -530,7 +627,6 @@ collinear <- function(x, rsq_cutoff = 0.9, verbose = FALSE) {
 #'   incorporated into all models are listed first. If `type = "full"` a matrix
 #'   of AIC values, sigma, the residual standard error (see [summary.lm]),
 #'   t-statistic and p-values for the tested predictor is returned.
-#' @importFrom RcppEigen fastLmPure
 #' @export
 #'
 lm_filter <- function(y, x,
@@ -538,7 +634,12 @@ lm_filter <- function(y, x,
                       nfilter = NULL,
                       p_cutoff = NULL,
                       rsq_cutoff = NULL,
+                      rsq_method = "pearson",
                       type = c("index", "names", "full")) {
+  if (!requireNamespace("RcppEigen", quietly = TRUE)) {
+    stop("Package 'RcppEigen' must be installed to use this filter",
+         call. = FALSE)
+  }
   type <- match.arg(type)
   if (is.data.frame(x)) {
     is_factor <- vapply(x, is.factor, logical(1))
@@ -554,7 +655,7 @@ lm_filter <- function(y, x,
   } else xset <- startx
   res <- sapply(check_vars, function(i) {
     xset[, 2] <- x[, i]
-    fit <- fastLmPure(xset, y, method = 3L)
+    fit <- RcppEigen::fastLmPure(xset, y, method = 3L)
     rss <- sum(fit$residuals^2)
     tval <- fit$coefficients[2] / fit$se[2]
     c(rss, tval)
@@ -575,7 +676,8 @@ lm_filter <- function(y, x,
   out <- out[order(out[,1]), ]
   if (!is.null(p_cutoff)) out <- out[out[, 'pval'] < p_cutoff, ]
   if (!is.null(rsq_cutoff)) {
-    co <- collinear(x[, rownames(out)], rsq_cutoff = rsq_cutoff)
+    co <- collinear(x[, rownames(out)], rsq_cutoff = rsq_cutoff,
+                    rsq_method = rsq_method)
     if (length(co) > 0) out <- out[-co, ]
   }
   if (!is.null(nfilter)) out <- out[1:min(nfilter, nrow(out)), ]
